//@ts-check
"use strict";

/**
@typedef FX
@type {{
  animSprite: PIXI.AnimatedSprite;
  sprite: PIXI.Sprite;
  update: Function;
}}
*/

function updateFxs(dt) {
  let i = 0;
  while (i < inGame.activeWorld.fxs.length) {
    const fx = inGame.activeWorld.fxs[i];

    if (fx.animSprite) {
      fx.animSprite.update(app.ticker.deltaTime);
      if (fx.animSprite.playing) {
        i++;
        continue;
      }

      fx.animSprite.destroy();
    }
    else {
      if (fx.update(fx, dt)) {
        i++;
        continue;
      }

      fx.sprite.destroy();
    }

    inGame.activeWorld.pixiLayers["ground"].particlesContainer.removeChild(fx.animSprite);
    inGame.activeWorld.fxs.splice(i, 1);
  }
}

/**
 * @param {string} spriteName
 * @param {PIXI.Point} pos
 * @param {number} angle
 * @param {string} animName
 * @param {PIXI.Container} container
 */
function spawnAnimatedFx(spriteName, pos, angle, animName = "anim", container = null) {
  let animSprite;

  const spritesheet = getSpritesheet(spriteName);
  animSprite = new PIXI.AnimatedSprite(spritesheet.animations[animName], false);
  animSprite.anchor.copyFrom(spritesheet.anchor);
  animSprite.scale.set(inverseTileSize, inverseTileSize);
  animSprite.position.copyFrom(pos);
  animSprite.angle = angle * PIXI.RAD_TO_DEG;

  if (container == null) container = inGame.activeWorld.pixiLayers["ground"].particlesContainer;
  container.addChild(animSprite);
  animSprite.loop = false;
  animSprite.play();

  /** @type {FX} */
  const fx = { sprite: null, animSprite, update: null };
  inGame.activeWorld.fxs.push(fx);
}

/**
 * @param {string} spriteName
 * @param {PIXI.Point} pos
 * @param {number} angle
 * @param {(fx: FX, dt: number) => void=} update
 */
function spawnFx(spriteName, pos, angle, update) {
  let sprite;

  sprite = new PIXI.Sprite(tex(spriteName));
  sprite.anchor.set(0.5);
  sprite.scale.set(inverseTileSize, inverseTileSize);
  sprite.position.copyFrom(pos);
  sprite.angle = angle * PIXI.RAD_TO_DEG;
  inGame.activeWorld.pixiLayers["ground"].particlesContainer.addChild(sprite);

  /** @type {FX} */
  const fx = { sprite, animSprite: null, update };
  inGame.activeWorld.fxs.push(fx);
}
